#インポート処理
require 'open-uri'
require 'common'

module ActiveRecord
  class Base
    module PettanImporter
      def self.included(base)
        base.extend(ClassMethods)
        base.__send__ :include, InstanceMethods
      end
      
      module ClassMethods
        def each_import data
          data.each do |n, d|
            yield n, d
          end
        end
        
        def import_system_picture attr
          d = attr["text"]
          return false if d.blank?
          imager = PettanImager.load(Base64.decode64(d.to_s))
          return false unless imager
          sp = SystemPicture.store(imager)
          return false unless sp
          sp.id
        end
        
        def replace_system_picture attr
          attr.each do |name, value|
            if value.is_a? Hash
              r = if name.to_s =~ /_id$/
                self.import_system_picture value
              else
                self.replace_system_picture value
              end
              return false unless r
              attr[name] = r
            end
          end
        end
        
        def modify_object(name, attr, key = 'name')
          c = 'find_by_' + key.to_s
          r = self.__send__ c, name
          if r
            r.attributes = attr
          else
            r = self.new attr
            r[key] = name
          end
          r
        end
        
        def import_list(attrs, &blk)
          res = []
          self.transaction do
            attrs.each do |name, item|
              m = blk.call(name, item)
              res.push(m) unless m.valid?
            end
            raise ActiveRecord::Rollback unless res.empty?
          end
          res
        end
        
        def import_text(text, &blk)
          attrs = JSON.parse(text)
          import_list attrs, &blk
        end
        
        def import_file(filename, &blk)
          text = File.open(filename, 'r').read
          self.import_text text, &blk
        end
        
        def import_url(url, &blk)
          text = open(url).read
          self.import_text text, &blk
        end
        
        def import_urls(urls, &blk)
          r = {}
          urls.each do |url|
            h = {}
            begin
              h[:validations] = import_url(url, &blk)
            rescue
              h[:exception] = {:location => $@, :message => $!}
            end
            r[url] = h
          end
          r
        end
        
      end
      
      module InstanceMethods
        private
        
        public
        
      end
      
    end
    include PettanImporter
  end
end

